@@ -3,6 +3,7 @@ use crate::delimiter::Delimiter;
 use crate::errors::CsvlensResult;
 use crate::io::SeekableFile;
 
+#[cfg(feature = "cli")]
 use clap::{command, Parser};
 use crossterm::execute;
 use crossterm::terminal::{
@@ -15,6 +16,7 @@ use std::io::LineWriter;
 use std::panic;
 use std::thread::panicking;
 
+#[cfg(feature = "cli")]
 #[derive(Parser, Debug)]
 #[command(version)]
 struct Args {
@@ -58,6 +60,39 @@ struct Args {
     debug: bool,
 }
 
+#[cfg(feature = "cli")]
+impl From<Args> for CsvlensOptions {
+    fn from(args: Args) -> Self {
+        Self {
+            filename: args.filename,
+            delimiter: args.delimiter,
+            tab_separated: args.tab_separated,
+            no_headers: args.no_headers,
+            columns: args.columns,
+            filter: args.filter,
+            find: args.find,
+            ignore_case: args.ignore_case,
+            echo_column: args.echo_column,
+            debug: args.debug,
+        }
+    }
+}
+
+// Struct for library usage without clap directives
+#[derive(Debug, Default)]
+pub struct CsvlensOptions {
+    pub filename: Option<String>,
+    pub delimiter: Option<String>,
+    pub tab_separated: bool,
+    pub no_headers: bool,
+    pub columns: Option<String>,
+    pub filter: Option<String>,
+    pub find: Option<String>,
+    pub ignore_case: bool,
+    pub echo_column: Option<String>,
+    pub debug: bool,
+}
+
 struct AppRunner {
     app: App,
 }
@@ -101,6 +136,50 @@ impl Drop for AppRunner {
     }
 }
 
+/// Run csvlens with options provided in a `CsvlensOptions` struct.
+///
+/// On success, the result contains an optional string that is the value of the selected cell if
+/// any. If csvlens exits without selecting a cell, the result is None.
+///
+/// Example:
+///
+/// ```
+/// use csvlens::{run_csvlens_with_options, CsvlensOptions};
+///
+/// let options = CsvlensOptions {
+///     filename: Some("/path/to/your.csv".to_string()),
+///     ..Default::default()
+/// };
+/// match run_csvlens_with_options(options) {
+///     Ok(Some(selected_cell)) => println!("Selected: {}", selected_cell),
+///     Ok(None) => {},
+///     Err(e) => eprintln!("Error: {:?}", e),
+/// }
+/// ```
+pub fn run_csvlens_with_options(options: CsvlensOptions) -> CsvlensResult<Option<String>> {
+    let show_stats = options.debug;
+    let delimiter = Delimiter::from_arg(&options.delimiter, options.tab_separated)?;
+
+    let file = SeekableFile::new(&options.filename)?;
+    let filename = file.filename();
+
+    let app = App::new(
+        filename,
+        delimiter,
+        options.filename,
+        show_stats,
+        options.echo_column,
+        options.ignore_case,
+        options.no_headers,
+        options.columns,
+        options.filter,
+        options.find,
+    )?;
+
+    let mut app_runner = AppRunner::new(app);
+    app_runner.run()
+}
+
 /// Run csvlens with a list of arguments. The accepted arguments are the same as the command line
 /// arguments for the csvlens binary.
 ///
@@ -118,6 +197,7 @@ impl Drop for AppRunner {
 ///     Err(e) => eprintln!("Error: {:?}", e),
 /// }
 /// ```
+#[cfg(feature = "cli")]
 pub fn run_csvlens<I, T>(args: I) -> CsvlensResult<Option<String>>
 where
     I: IntoIterator<Item = T>,
@@ -128,26 +208,15 @@ where
         args_items.push(item.into());
     }
     let args = Args::parse_from(args_items);
-
-    let show_stats = args.debug;
-    let delimiter = Delimiter::from_arg(&args.delimiter, args.tab_separated)?;
-
-    let file = SeekableFile::new(&args.filename)?;
-    let filename = file.filename();
-
-    let app = App::new(
-        filename,
-        delimiter,
-        args.filename,
-        show_stats,
-        args.echo_column,
-        args.ignore_case,
-        args.no_headers,
-        args.columns,
-        args.filter,
-        args.find,
-    )?;
-
-    let mut app_runner = AppRunner::new(app);
-    app_runner.run()
+    run_csvlens_with_options(args.into())
+}
+
+#[cfg(not(feature = "cli"))]
+pub fn run_csvlens<I, T>(_args: I) -> CsvlensResult<Option<String>>
+where
+    I: IntoIterator<Item = T>,
+    T: Into<OsString> + Clone,
+{
+    eprintln!("Error: CLI is not enabled. Compile with the 'cli' feature to use this binary.");
+    std::process::exit(1);
 }
