@@ -12,6 +12,7 @@ use ratatui::prelude::Position;
 use ratatui::style::{Color, Modifier, Style};
 use ratatui::symbols::line;
 use ratatui::text::{Line, Span};
+use ratatui::widgets::Clear;
 use ratatui::widgets::Widget;
 use ratatui::widgets::{Block, Borders, StatefulWidget};
 use regex::Regex;
@@ -125,7 +126,7 @@ impl<'a> CsvTable<'a> {
         }
 
         // If clipping was too aggressive, redistribute the remaining width
-        CsvTable::redistribute_widths_after_clpping(
+        CsvTable::redistribute_widths_after_clipping(
             &mut column_widths,
             area_width,
             clipped_columns,
@@ -134,7 +135,7 @@ impl<'a> CsvTable<'a> {
         column_widths
     }
 
-    fn redistribute_widths_after_clpping(
+    fn redistribute_widths_after_clipping(
         column_widths: &mut [u16],
         area_width: u16,
         mut clipped_columns: Vec<(usize, u16)>,
@@ -257,6 +258,7 @@ impl<'a> CsvTable<'a> {
         state.borders_state = Some(BordersState {
             x_row_separator: view_layout.row_number_layout.x_row_separator,
             y_first_record,
+            x_freeze_separator: view_layout.x_freeze_separator,
         });
     }
 
@@ -286,6 +288,7 @@ impl<'a> CsvTable<'a> {
             return;
         }
 
+        // Line number separator
         let line_number_block = Block::default()
             .borders(Borders::RIGHT)
             .border_style(Style::default().fg(Color::Rgb(64, 64, 64)));
@@ -339,6 +342,39 @@ impl<'a> CsvTable<'a> {
                     .set_symbol(line::HORIZONTAL_UP);
             }
         }
+
+        // Freeze separator
+        if let Some(x_freeze_separator) = borders_state.x_freeze_separator {
+            // Clear highlight style made by render_row before rendering the separator
+            if x_freeze_separator < area.right() {
+                let x_freeze_separator_area =
+                    Rect::new(x_freeze_separator, y_first_record, 1, area.height);
+                Clear.render(x_freeze_separator_area, buf);
+
+                if let Some(cell) = buf.cell_mut(Position::new(
+                    x_freeze_separator,
+                    y_first_record.saturating_sub(1),
+                )) {
+                    cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
+                        .set_symbol("╥");
+                }
+
+                for y in y_first_record..y_first_record + area.height {
+                    if let Some(cell) = buf.cell_mut(Position::new(x_freeze_separator, y)) {
+                        cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
+                            .set_symbol(line::DOUBLE_VERTICAL);
+                    }
+                }
+
+                if let Some(cell) = buf.cell_mut(Position::new(
+                    x_freeze_separator,
+                    y_first_record + area.height,
+                )) {
+                    cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
+                        .set_symbol("╨");
+                }
+            }
+        }
     }
 
     fn get_effective_column_name(&self, column_name: &str, sorter_state: &SorterState) -> String {
@@ -371,7 +407,6 @@ impl<'a> CsvTable<'a> {
     ) -> u16 {
         let mut x_offset_header = x;
         let mut remaining_width = area.width.saturating_sub(x);
-        let cols_offset = state.cols_offset as usize;
         // TODO: seems strange that these have to be set every row
         let mut has_more_cols_to_show = false;
         let mut col_ending_pos_x = 0;
@@ -384,7 +419,10 @@ impl<'a> CsvTable<'a> {
             },
         };
         for (col_index, (hname, &hlen)) in row.iter().zip(column_widths).enumerate() {
-            if col_index < cols_offset {
+            if !state
+                .cols_offset
+                .should_filtered_column_index_be_rendered(col_index as u64)
+            {
                 continue;
             }
             let effective_width = min(remaining_width, hlen);
@@ -421,25 +459,40 @@ impl<'a> CsvTable<'a> {
             };
 
             let should_highlight_cell = |active: &FinderActiveState, content: &str| {
+                // Only highlight the selected column in column selection mode. But header search is
+                // always across all columns regardless of the selection mode.
                 if let Some((target_column_index, _)) = active.column_index {
-                    if target_column_index != col_index {
+                    if target_column_index != col_index && matches!(row_type, RowType::Record(_)) {
                         return false;
                     }
                 }
-                active.target.is_match(content) && !matches!(row_type, RowType::Header)
+                if active.is_filter && matches!(row_type, RowType::Header) {
+                    return false;
+                }
+                active.target.is_match(content)
             };
             match &state.finder_state {
                 // TODO: seems like doing a bit too much of heavy lifting of
                 // checking for matches (finder's work)
                 FinderState::FinderActive(active) if should_highlight_cell(active, hname) => {
                     let mut highlight_style = filler_style.style.fg(Color::Rgb(200, 0, 0));
-                    if let Some(hl) = &active.found_record {
-                        if let Some(row_index) = row_index {
-                            // TODO: vec::contains slow or does it even matter?
-                            if row_index == hl.row_index()
-                                && hl.column_indices().contains(&col_index)
-                            {
-                                highlight_style = highlight_style.bg(Color::LightYellow);
+                    if let Some(found_record) = &active.found_record {
+                        match found_record {
+                            find::FoundEntry::Row(entry) => {
+                                if let Some(row_index) = row_index {
+                                    if row_index == entry.row_index()
+                                        && entry.column_index() == col_index
+                                    {
+                                        highlight_style = highlight_style.bg(Color::LightYellow);
+                                    }
+                                }
+                            }
+                            find::FoundEntry::Header(entry) => {
+                                if matches!(row_type, RowType::Header)
+                                    && entry.column_index() == col_index
+                                {
+                                    highlight_style = highlight_style.bg(Color::LightYellow);
+                                }
                             }
                         }
                     }
@@ -484,7 +537,7 @@ impl<'a> CsvTable<'a> {
             remaining_width = remaining_width.saturating_sub(hlen);
         }
         state.num_cols_rendered = max(state.num_cols_rendered, num_cols_rendered);
-        state.set_more_cols_to_show(has_more_cols_to_show);
+        state.update_more_cols_to_show(has_more_cols_to_show);
         state.col_ending_pos_x = max(state.col_ending_pos_x, col_ending_pos_x);
         row_height
     }
@@ -629,6 +682,7 @@ impl<'a> CsvTable<'a> {
                     InputMode::Filter => "Filter",
                     InputMode::FilterColumns => "Columns regex",
                     InputMode::Option => "Option",
+                    InputMode::FreezeColumns => "Number of columns to freeze",
                     _ => "",
                 };
                 if prefix.is_empty() {
@@ -678,7 +732,7 @@ impl<'a> CsvTable<'a> {
                 " [Row {}/{}, Col {}/{}]",
                 row_num,
                 total_str,
-                state.cols_offset + 1,
+                state.cols_offset.num_skip + 1,
                 state.total_cols,
             )
             .as_str();
@@ -752,10 +806,25 @@ impl<'a> CsvTable<'a> {
             width_with_spaces: row_num_section_width_with_spaces,
             x_row_separator,
         };
+
+        // x-position of the vertical separator if any columns are frozen
+        let x_freeze_separator = if state.cols_offset.num_freeze > 0 {
+            let mut x_sum = row_number_layout.x_row_separator;
+            for (column_index, width) in column_widths.iter().enumerate() {
+                if state.cols_offset.is_frozen(column_index as u64) {
+                    x_sum += width;
+                }
+            }
+            Some(x_sum)
+        } else {
+            None
+        };
+
         ViewLayout {
             column_widths,
             row_heights,
             row_number_layout,
+            x_freeze_separator,
         }
     }
 }
@@ -790,6 +859,7 @@ impl StatefulWidget for CsvTable<'_> {
         self.render_row_numbers(buf, state, rows_area, self.rows, &layout);
         let row_num_section_width = layout.row_number_layout.width_with_spaces;
 
+        state.reset_more_cols_to_show();
         self.render_row(
             buf,
             state,
@@ -869,6 +939,7 @@ pub struct ViewLayout {
     pub column_widths: Vec<u16>,
     pub row_heights: Vec<u16>,
     pub row_number_layout: RowNumberLayout,
+    pub x_freeze_separator: Option<u16>,
 }
 
 impl ViewLayout {
@@ -910,27 +981,31 @@ impl FinderState {
 pub struct FinderActiveState {
     find_complete: bool,
     total_found: u64,
-    cursor_index: Option<u64>,
+    cursor: Option<find::FinderCursor>,
     target: Regex,
     column_index: Option<(usize, String)>,
-    found_record: Option<find::FoundRecord>,
+    found_record: Option<find::FoundEntry>,
     selected_offset: Option<u64>,
     is_filter: bool,
+    header_has_match: bool,
 }
 
 impl FinderActiveState {
     pub fn new(finder: &find::Finder, rows_view: &view::RowsView) -> Self {
+        let header_has_match = finder.header_has_match();
+        let total_count = finder.count() + if header_has_match { 1 } else { 0 };
         FinderActiveState {
             find_complete: finder.done(),
-            total_found: finder.count() as u64,
-            cursor_index: finder.cursor().map(|x| x as u64),
+            total_found: total_count as u64,
+            cursor: finder.cursor(),
             target: finder.target(),
             column_index: finder
                 .column_index()
-                .map(|i| (i, rows_view.get_column_name_from_global_index(i))),
+                .map(|i| (i, rows_view.get_column_name_from_local_index(i))),
             found_record: finder.current(),
             selected_offset: rows_view.selected_offset(),
             is_filter: rows_view.is_filter(),
+            header_has_match,
         }
     }
 
@@ -956,8 +1031,14 @@ impl FinderActiveState {
                 } else {
                     cursor_str = "-".to_owned();
                 }
-            } else if let Some(i) = self.cursor_index {
-                cursor_str = (i.saturating_add(1)).to_string();
+            } else if let Some(cursor) = &self.cursor {
+                cursor_str = match cursor.row {
+                    find::RowPos::Row(i) => (i
+                        .saturating_add(1)
+                        .saturating_add(if self.header_has_match { 1 } else { 0 }))
+                    .to_string(),
+                    find::RowPos::Header => "1".to_string(),
+                };
             } else {
                 cursor_str = "-".to_owned();
             }
@@ -1050,6 +1131,7 @@ impl SorterInfo {
 struct BordersState {
     x_row_separator: u16,
     y_first_record: u16,
+    x_freeze_separator: Option<u16>,
 }
 
 pub struct DebugStats {
@@ -1120,9 +1202,9 @@ impl DebugStats {
 pub struct CsvTableState {
     // TODO: types appropriate?
     pub rows_offset: u64,
-    pub cols_offset: u64,
+    pub cols_offset: view::ColumnsOffset,
     pub num_cols_rendered: u64,
-    pub more_cols_to_show: bool,
+    pub more_cols_to_show: Option<bool>,
     filename: Option<String>,
     total_line_number: Option<(usize, bool)>,
     total_cols: usize,
@@ -1155,9 +1237,9 @@ impl CsvTableState {
     ) -> Self {
         Self {
             rows_offset: 0,
-            cols_offset: 0,
+            cols_offset: view::ColumnsOffset::default(),
             num_cols_rendered: 0,
-            more_cols_to_show: true,
+            more_cols_to_show: None,
             filename,
             total_line_number: None,
             total_cols,
@@ -1185,16 +1267,25 @@ impl CsvTableState {
         self.rows_offset = offset;
     }
 
-    pub fn set_cols_offset(&mut self, offset: u64) {
+    pub fn set_cols_offset(&mut self, offset: view::ColumnsOffset) {
         self.cols_offset = offset;
     }
 
-    fn set_more_cols_to_show(&mut self, value: bool) {
-        self.more_cols_to_show = value;
+    fn reset_more_cols_to_show(&mut self) {
+        self.more_cols_to_show = None;
+    }
+
+    fn update_more_cols_to_show(&mut self, value: bool) {
+        // If any rows have more columns to show, keep it that way
+        if let Some(current) = self.more_cols_to_show {
+            self.more_cols_to_show = Some(current || value);
+        } else {
+            self.more_cols_to_show = Some(value);
+        }
     }
 
     pub fn has_more_cols_to_show(&self) -> bool {
-        self.more_cols_to_show
+        self.more_cols_to_show.map_or(true, |v| v)
     }
 
     pub fn set_total_line_number(&mut self, n: usize, is_approx: bool) {
